"{ Package: 'jv:libgdbs' }"

GDBProcess subclass:#GDBSimulatorProcess
	instanceVariableNames:'record thread steppingSemaphore'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Private-Simulator'
!

!GDBSimulatorProcess methodsFor:'execution'!

pause
    steppingSemaphore isNil ifTrue:[ 
        steppingSemaphore := Semaphore new.
    ] ifFalse:[
        steppingSemaphore clear.        
    ].

    "Created: / 24-06-2014 / 13:05:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

resume
    | sema |

    steppingSemaphore notNil ifTrue:[ 
        sema :=  steppingSemaphore.
        steppingSemaphore := nil.
        sema signalForAll.
    ].
    thread isNil ifTrue:[ self start ].

    "Created: / 24-06-2014 / 13:05:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

step
    steppingSemaphore isNil ifTrue:[ 
        steppingSemaphore := Semaphore new.
    ].
    thread isNil ifTrue:[ self start ].
    steppingSemaphore signal.

    "Created: / 24-06-2014 / 13:02:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBSimulatorProcess methodsFor:'initialization'!

initialize
    pid := -1.
    debuggerInput := GDBInternalPipeStream new.
    debuggerOutput := GDBInternalPipeStream new.

    "Modified: / 20-06-2014 / 22:18:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBSimulatorProcess methodsFor:'processing'!

processCommand: actual expecting: expected

    actual class == GDBMI_gdb_exit ifTrue:[ 
        actual token notNil ifTrue:[ 
            debuggerOutput nextPutAll: actual token printString.
        ].
        debuggerOutput nextPutLine: '^exit'.
        ^ true.
    ].
    (actual class == GDBMI_inferior_tty_set and:[expected class == GDBMI_inferior_tty_set]) ifTrue:[ 
        ^ false.
    ].

    actual token notNil ifTrue:[ 
        debuggerOutput nextPutAll: actual token printString.
    ].    
    debuggerOutput nextPutLine: '^error,msg="Unexpected command"'.
    ^ false

    "Created: / 24-06-2014 / 23:37:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

processCommandLineExpecting: lineExpected
    | lineActual |

    lineActual := debuggerInput nextLine.
    lineActual ~= lineExpected ifTrue:[ 
        | cmdActual cmdExpected |

        cmdActual := (GDBParser on: lineActual ) parseCommand.  
        cmdExpected := (GDBParser on: lineExpected ) parseCommand.  

        ^ self processCommand: cmdActual expecting: cmdExpected.  
    ].
    ^ false

    "Created: / 24-06-2014 / 23:37:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

processLoop
    | done events |

    done := false.
    events := record readStream.
    [ done or:[ events atEnd ] ] whileFalse:[
        | event |

        event := events next.
        event isCommand ifTrue:[ 
            done := self processCommandLineExpecting: event string.
        ] ifFalse:[
            self processResponse: event.
        ]

    ].

    "Created: / 24-06-2014 / 08:48:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 24-06-2014 / 23:37:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

processResponse: response
    debuggerOutput nextPutAll: response string

    "Created: / 24-06-2014 / 22:57:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBSimulatorProcess methodsFor:'start & stop'!

record
    ^ record
!

record:aGDBSessionRecord
    record := aGDBSessionRecord.

    "Modified: / 24-06-2014 / 13:06:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

start
    self stop.

    thread := [ 
        self processLoop.
    ] newProcess.
    thread addExitAction: [ thread := nil ].
    thread resume.

    "Created: / 24-06-2014 / 00:58:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 24-06-2014 / 08:48:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

stop
    (thread notNil and:[ thread isDead not ]) ifTrue:[  
        thread terminate.
        thread := nil.
    ]

    "Created: / 24-06-2014 / 01:02:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

